# ----------------------------------------------------------------------
#  EXAMPLE: GAMESS function in Tcl.
#
#  This simple example shows how to use Rappture within a simulator
#  written in Tcl.
# ======================================================================
#  AUTHOR:  Baudilio Tejerina. Northwestern University
#  Copyright (c) 2004-2005
#  Northwestern University, Evanston 60308 IL
# ======================================================================

package require BLT

proc GMS_exec {args} {
    variable execout
    variable execctl

    set execout(output) ""
    set execout(channel) ""
    set execout(extra) ""
    set execctl ""

    set status [catch {eval blt::bgexec ::Rappture::execctl \
        -keepnewline yes \
        -killsignal SIGTERM \
        -onoutput {{GMS_exec_out stdout}} \
        -onerror  {{GMS_exec_out stderr}} \
        $args} result]

    # add any extra stuff pending from the last stdout/stderr change
    append execout(output) $execout(extra)

    if {$status != 0} {
        if {[regexp {^KILLED} $execctl] && [llength $execctl] == 4} {
            set reason [lindex $execctl end]
            set result "job killed: $reason"
        }
        error $result
    }
    return $execout(output)
}

proc GMS_exec_out {channel message} {
    variable execout

    #
    # If this message is being written to the stderr channel, then
    # add the =RAPPTURE-ERROR=> prefix to each line.
    #
    if {$channel == "stderr"} {
        set newmesg ""
        foreach line [split $message \n] {
            append newmesg "=RAPPTURE-ERROR=>$line\n"
        }
        set message $newmesg
    }
    # Look for a pattern and figure out progress.
    if {[regexp {GEOMETRY SEARCH POINT NSERCH= ([0-9]+)} $message match number]} {
        puts "=RAPPTURE-PROGRESS=>$number found Optimization Step..."
    }

    #
    # If this message is coming in on the same channel as the
    # last, then fine, add it on.  But if it's coming in on a
    # different channel, we must make sure that we're at a good
    # breakpoint.  If there's not a line break at the end of the
    # current output, then add the extra stuff onto a buffer
    # that we will merge in later once we get to a good point.
    #
    if {$execout(channel) == ""} {
        set execout(channel) $channel
    }

    set ready [expr {[string length $execout(output)] == 0
        || [string index $execout(output) end] == "\n"}]

    if {$channel != $execout(channel)} {
        if {$ready} {
            # changing channels...
            if {[string length $execout(extra)] > 0} {
                # any extra stuff on this new channel? put it out now
                puts -nonewline $execout(extra)
                append execout(output) $execout(extra)
                set execout(extra) ""
            }
            puts -nonewline $message
            append execout(output) $message
            set execout(channel) $channel
        } else {
            # not ready to change channels -- keep this for later
            append execout(extra) $message
        }
    } else {
        # no need to change channels -- keep printing
        puts -nonewline $message
        append execout(output) $message
    }
}



package require Rappture

# open the XML file containing the run parameters
set driver [Rappture::library [lindex $argv 0]]


set JOB    [$driver get input.(Job).current]
set RUNTYP [$driver get input.(TheoMod).(CONTRL).(RunTyp).current]
set EXETYP [$driver get input.(TheoMod).(CONTRL).(ExeTyp).current]

set TIMLIM [$driver get input.(TheoMod).(CONTRL).(TIMLIM).current]
set NPROCS [$driver get input.(TheoMod).(CONTRL).(NPROCS).current]
set MWORDS [$driver get input.(TheoMod).(CONTRL).(MWORDS).current]
set MEMDDI [$driver get input.(TheoMod).(CONTRL).(MEMDDI).current]


# Memory  Per Node:
set MPN [expr {$MWORDS + $MEMDDI/$NPROCS}]
set MPN [expr {8*$MPN/1024}]; # Giga Bytes. We can use this number to select the 'venue'.


set SCFTYP [$driver get input.(TheoMod).(CONTRL).(SCFTyp).current]
set ICHARG [$driver get input.(TheoMod).(CONTRL).(Charge).current]
set MULT   [$driver get input.(TheoMod).(CONTRL).(Spin).current]


#Process additional commands the user might need: (Specific Job Instructions) SJI.
set EXTCOM [$driver get input.(TheoMod).(CONTRL).(SJI).current]
set Extra_Command [lrange [split $EXTCOM {$}] 1 end-1 ]
#

#BASIS Set
set BS_Kind  [$driver get input.(TheoMod).(BASIS).(KIND).current]

switch $BS_Kind {
    "AllE" {
	set BASIS  [$driver get input.(TheoMod).(BASIS).(AllElectrons).(Basis).current]
    }
    "ECP" {
 	set BASIS  [$driver get input.(TheoMod).(BASIS).(ECP).(Basis).current]
    }
    "SemiEmp" {
	set BASIS  [$driver get input.(TheoMod).(BASIS).(Semiempirical).(Basis).current]
    }
default {set BASIS "STO-3G"}
}

#

set METHOD [$driver get input.(TheoMod).(POSTSCF).(METHOD).current]
set DFTTyp None
set MPLevl 0


if { $METHOD == "AbIn" } {
    set POSTSCF_ab   [$driver get input.(TheoMod).(POSTSCF).(AbInitio).current]
    
    switch $POSTSCF_ab {
	"None: HF"                      {set MPLevl 0}
	"2nd-Order Moller-Plesset: MP2" {set MPLevl 2}
	default {set MPLevl 0}
    }
    set DFTTyp None

} else {
    set POSTSCF_dft  [$driver get input.(TheoMod).(POSTSCF).(DFT).current]
    
    switch $POSTSCF_dft {
	"None (HF)" {set DFTTyp None}
	"B3LYP"     {set DFTTyp B3LYP}
	"BHHLYP"    {set DFTTyp BHHLYP}
	default {set DFTTyp None}
    }
    set MPLevl 0
}


#Cartesian coordinates of the Molecule. 
set Atomic_Coordinates [$driver get input.(DATA).(xyz).current]
set Point_Group        [$driver get input.(DATA).(PointGroup).current]
set Symmetry_Order     [$driver get input.(DATA).(SymmOrder).current]
set Coordinates_Type   [$driver get input.(DATA).(Coord).current]
set Units              [$driver get input.(DATA).(Units).current]
#set BA 1.0; if {$Units == "Bohrs"} { set BA 0.529177249}; # Bohrs to Angstroms.

# 
set REC  [split $Atomic_Coordinates \n]
set NREC [llength $REC]
set Atomic_Coordinates ""; #BTA let's reset it to fill it with 'well-formatted' coords

set NAT 0;
for {set i 0} {$i <= $NREC} {incr i} {
    set line [lindex $REC $i]
    set SCAN_IO [scan $line {%s %g %g %g %g} Sym AN x y z ]
    if {$SCAN_IO != -1} {
	incr NAT
	append Atomic_Coordinates [format "%s\ %5.1f\ %10.5f\ %10.5f\ %10.5f%s" $Sym $AN $x $y $z \n]
    }
}

#puts "BTA>>>>   $Atomic_Coordinates"

if { $Coordinates_Type == "Cartesian" } { set Coordinates_Type "Cart" }

if { $Point_Group == "T"  || $Point_Group == "Td" || \
     $Point_Group == "Th" || $Point_Group == "O"  || \
     $Point_Group == "Oh" || \
     $Point_Group == "Ci" || $Point_Group == "Cs" }  { set Symmetry_Order  "" } 

if { $Point_Group == "C_inf_v" } { set Point_Group "Cnv"; set Symmetry_Order 4 } 
if { $Point_Group == "D_inf_h" } { set Point_Group "Dnh"; set Symmetry_Order 4 } 


#Solvent Models:
set Solvent_Model [$driver get input.(TheoMod).(Solvent).(MODEL).current]
#COSMO
set EPSI 0.0
if {$Solvent_Model == "COSMO"} {
 set EPSI [$driver get input.(TheoMod).(Solvent).(COSMO).(EPSI).current]
}

#PCM
set PCM_Solvnt Vacuum
if {$Solvent_Model == "PCM"} {
 set SOLVENT [$driver get input.(TheoMod).(Solvent).(PCM).(PCM).current]

 switch $SOLVENT {
    "None (Vacuum)"              {set PCM_Solvnt Vacuum}
    "WATER [H2O]"                {set PCM_Solvnt H2O}
    "Methanol [CH3OH]"           {set PCM_Solvnt CH3OH}
    "Ethanol [C2H5OH]"           {set PCM_Solvnt C2H5OH}
    "Chloroform [CHCl3]"         {set PCM_Solvnt CHCl3}
    "Tetrachloromethane [CCl4]"  {set PCM_Solvnt CCl4}
    "Dichloromethane [CH2Cl2]"   {set PCM_Solvnt CH2Cl2}
    "Dichloroethane [C2H4Cl2]"   {set PCM_Solvnt C2H4Cl2}
    "Benzene [C6H6]"             {set PCM_Solvnt C6H6}
    "Toluene [C6H5CH3]"          {set PCM_Solvnt C6H5CH3}
    "Chlorobenzene [C6H5Cl]"     {set PCM_Solvnt C6H5Cl}
    "Nitromethane [CH3NO2]"      {set PCM_Solvnt CH3NO2}
    "n-Heptane [C7H16]"          {set PCM_Solvnt C7H16}
    "Cyclohexane [C6H12]"        {set PCM_Solvnt C6H12}
    "Aniline [C6H5NH2]"          {set PCM_Solvnt C6H5NH2}
    "Acetone [CH3COCH3]"         {set PCM_Solvnt CH3COCH3}
    "Tetrahydrofurane [THF]"     {set PCM_Solvnt THF}
    "Dimethylsulfoxide [DMSO]"   {set PCM_Solvnt DMSO}
            default {set PCM_Solvnt Vacuum}
 }

}

#
set HssEnd ".f."
switch $RUNTYP {
    "Single Point: Energy Minimization" {set RunTyp "Energy"}
    "Hessian of the Molecular Energy"   {set RunTyp "Hessian"}
    "Geometry Optimization"             {set RunTyp "Optimize"}
    "Geometry Optimization + Hessian"   {set RunTyp "Optimize"; set HssEnd ".t."}
           default {set RunTyp "Optimize"}
}


switch $SCFTYP {
    "Restricted Hartree-Fock Calculation"   {set SCFTyp "RHF"}
    "Unrestricted Hartree-Fock Calculation" {set SCFTyp "UHF"}
    "Restricted Open Shell Hartree-Fock Calculation" {set SCFTyp "ROHF"}
    default {set SCFTyp "RHF"}
}

#set NGauss ""

switch $BASIS {
    "MINI"   {set GBasis "MINI"}
    "MIDI"   {set GBasis "MIDI"}
    "DZV"    {set GBasis "DZV" }
    "DZVP"   {set GBasis "DZV nPFunc=1 nDFunc=1"}
    "TZV"    {set GBasis "TZV" }
    "TZVP"   {set GBasis "TZV nPFunc=1 nDFunc=1"}

    "SBKJC"  {set GBasis "SBKJC"}
    "HW"     {set GBasis "HW"   }

    "MNDO"   {set GBasis "MNDO"}
    "AM1"    {set GBasis "AM1" }
    "PM3"    {set GBasis "PM3" }

    "STO-3G" {set GBasis "STO  NGauss= 3"}
    "6-31G"  {set GBasis "N31  NGauss= 6"}
    "6-311G" {set GBasis "N311 NGauss= 6"}
    default  {set GBasis "STO  NGauss= 3"}
}

# If the system is not singlet....
#if {$MULT > 1} {set SCFTyp "ROHF"}


# PROPERTIES SECTION:
set PROPERTIES [$driver get input.(PROPERTIES).(CALC).current]
if {$PROPERTIES == "yes"} {
    set SCFTyp== "RHF";set MPLevl 0;set DFTTyp None
    set RunTyp "TDHF";
    set NFREQ [$driver get input.(PROPERTIES).(TDHF).(NFREQ).current]
    set PROP_KEYS "   NFreq= $NFREQ\n"
    set  FREQ [$driver get input.(PROPERTIES).(TDHF).(FREQ).current]
    append PROP_KEYS "   FREQ(1)= $FREQ\n"
    set  INIB [$driver get input.(PROPERTIES).(TDHF).(INIB).current]
    if {$INIB != 0} {
    append PROP_KEYS "   INIB= $INIB"
	set  BSHG [$driver get input.(PROPERTIES).(TDHF).(BETAS).(BSHG).current]
	if {$BSHG == "yes"}  {append PROP_KEYS " BSHG=.t."}
	set  BEOPE [$driver get input.(PROPERTIES).(TDHF).(BETAS).(BEOPE).current]
	if {$BEOPE == "yes"} {append PROP_KEYS " BEOPE=.t."}
	set  BOR  [$driver get input.(PROPERTIES).(TDHF).(BETAS).(BOR).current]
	if {$BOR  == "yes"}  {append PROP_KEYS " BOR=.t."}
    }
    set  INIG [$driver get input.(PROPERTIES).(TDHF).(INIG).current]
    if {$INIG != 0} {
    append PROP_KEYS "\n   INIG= $INIG"
	set  GTHG    [$driver get input.(PROPERTIES).(TDHF).(GAMMAS).(GTHG).current]
	if {$GTHG == "yes"}   {append PROP_KEYS " GTHG=.t."}
	set  GEFISH  [$driver get input.(PROPERTIES).(TDHF).(GAMMAS).(GEFISH).current]
	if {$GEFISH == "yes"} {append PROP_KEYS " GEFISH=.t."}
	set  GIDRI   [$driver get input.(PROPERTIES).(TDHF).(GAMMAS).(GIDRI).current]
	if {$GIDRI == "yes"}  {append PROP_KEYS " GIDRI=.t."}
	set  GOKE    [$driver get input.(PROPERTIES).(TDHF).(GAMMAS).(GOKE).current]
	if {$GOKE == "yes"}   {append PROP_KEYS " GOKE=.t."}
    }
}
# END PROPERTIES SECTION

#Prepare the input file:
set deck    " \$CONTRL\n"
append deck "   ExeTyp=$EXETYP RunTyp=$RunTyp SCFTyp=$SCFTyp MPLevl=$MPLevl\n" 
if {$GBasis == "SBKJC"} {append deck "     ECP=SBKJC\n"}
if {$GBasis == "HW"}    {append deck "     ECP=HW\n"}
append deck "   iCharg=$ICHARG Mult=$MULT\n"
append deck "   PltOrb=.T. NPrint=9 AIMPAC=.T. MaxIt=120 \n"
append deck "   Coord=$Coordinates_Type \n"
if {$Units == "Bohrs"} { append deck "   Units=BOHR \n"}
append deck " \$END\n"

append deck " \$SCF DirSCF=.T.
      DIIS=.t. SOSCF=.f.
      SWDIIS=0.01 DEMCut=0.01 DMPCut=0.01
 \$END\n"

#Memory Config:
#DDI is only used if MPLev=2 (MP2 calculation).
#Also, if SCFTyp=MCSCF with FullNR...
if {$MPLevl == 2} {
append deck " \$SYSTEM Parall=.T. MWords=$MWORDS MemDDI=$MEMDDI \$END\n"
} else {
append deck " \$SYSTEM Parall=.T. MWords=$MWORDS \$END\n"
}

#DFT Group:
if {$DFTTyp != "None"} {append deck " \$DFT DFTTyp=$DFTTyp \$END\n"}

#Basis Set
append deck " \$BASIS GBasis=$GBasis \$END\n"
#We don't need this anymore:
#if {[string length $NGauss]==0} {
#    append deck " \$BASIS GBasis=$GBasis \$END\n"
#} else {
#    append deck " \$BASIS GBasis=$GBasis NGauss=$NGauss \$END\n"    
#}


append deck " \$STATPT NStep=60 OptTol=1.0E-4 HssEnd=$HssEnd \$END\n\n"


# < Add $Solvnt Group, if needed>
if {$PCM_Solvnt   != "Vacuum"} { append deck " \$PCM Solvnt=$PCM_Solvnt \$END\n\n"}
#if {$COSMO_Solvnt != "Vacuum"} { append deck " \$COSGMS EPSI=$EPSI \$END\n"}
if {$EPSI != 0.0} { append deck " \$COSGMS EPSI=$EPSI \$END\n\n"}

if {$PROPERTIES == "yes"} {
    append deck " \$TDHF\n"
    append deck "$PROP_KEYS\n"
    append deck " \$END\n"
    append deck " \$SCF    DIRSCF=.F. \$END\n"
    append deck " \$CONTRL NoSym=1 \$END\n\n"
}

#   } elseif {[string compare $DFTTyp "None"] != 1 } {
#The $DATA group:
append deck " \$DATA\n"
#Add Title
if {$MPLevl == 2} {
       append deck " $JOB: MP2/$BASIS Theoretical Level\n"
   } elseif {$DFTTyp != "None" } {
       append deck " $JOB: DFT{$DFTTyp}/$BASIS Theoretical Level\n"
   } else {
       append deck " $JOB: $SCFTyp/$BASIS Theoretical Level\n"
   }
#
append deck " $Point_Group  $Symmetry_Order\n"
append deck "\n"
append deck "$Atomic_Coordinates"
append deck " \$END\n"

#Add user specific command:
for {set i 0} {$i < [llength $Extra_Command]} {incr i 2} {
    append deck " \$[lindex $Extra_Command $i] \$END\n"
}



#Write INPUT file
regsub -all { } $JOB "_" JOB
set JOB "$JOB[pid]"
set inp [open $JOB.inp w]
puts $inp $deck
close $inp

#
# Execute the job.
# This used to check to see if a job already existed, but that's
# dangerous.  A user might change one parameter (not the job name)
# and yet we really would need to run again.  Rappture will handle
# the case of looking for existing data later.  For now, we execute
# every time the user asks.
#


# RUN GAMESS
#SGE@Europa:
#set status [catch {Rappture::exec /usr/local/gamess/gms_sge -p 1 -l $JOB.out -w 1:00:00  $JOB} info]

# 1.- Interactive@Europa:
##set status [catch {Rappture::exec /usr/local/gamess/rungms $JOB} info]
#
# 2.- NanoHUB workspace (developing)
##set status [catch {Rappture::exec /home/nanohub/tejerina/bin/rungms $JOB} info]
# 0.- Actual execution script @NnaoHUB
#set status [catch {Rappture::exec rungms $JOB} info]
#--- set status [catch {GMS_exec rungms $JOB} info]


# set NPROCS 4;
#set TIMLIM  24:00:00; # time to pass to 'submit'. See also time on tools.xml
# set TIMLIM  14400; # time must be passed in minutes -> (14400) = 10 days.
# set TIMLIM    240; # 240 min -> 4 hour.

if {[string compare $BS_Kind "SemiEmp"] == 0 } {set NPROCS 1}
if {$EXETYP == "Check"} {set NPROCS 1}

puts "The Job $JOB ... \nhas been submitted for execution.\n\n"
if {$NPROCS == 1} {
    puts "\tEither by user's request or because of certain requirements of the theoretical mode,\n\tthis job will be running in sequential mode: $NPROCS processor."
    puts "\t The job will enter in execution immediately depending on the availability of the processor."
    set status [catch {Rappture::exec submit --local rungms $JOB.inp 01} LOG]
} else {
    puts "\tIt will run in parallel using $NPROCS processors during $TIMLIM minutes.\n"
    set status [catch {Rappture::exec submit --run $JOB -w $TIMLIM -n $NPROCS \
			   gamess-20130501 $JOB.inp 01 $NPROCS} LOG]
}

# NOTE: LOG contains the output from 'submit' not from GAMESS

#Check out whether it has been any problem in the submission or the calculation:
#if {$status != 0 || [regexp {Execution terminated} $LOG]} {
if {$status != 0 } {

# Let's recover data from $LOG, if any.

#do this instead:
set path "output.log(INFOERR)"
set MESSAGE "BTA: Error sending the job to execution.\n"
$driver put -append yes $path $MESSAGE
$driver put -append yes $path "LOG: $LOG";
set path "output.log(INFOERR).about.label"
$driver put -append yes $path "Incomplete RUN";
set path "output.log(INFOERR).about.description"
$driver put -append yes $path "This section contains the repot for an incomplete job and the reasons for the carsh.\n";

# clean up files...
foreach name [glob -nocomplain $JOB.*] {
    file delete -force $name
}

Rappture::result $driver    
exit 0; # or 1
}

# Otherwise, The job was accepted by the batch manager and queued for executuon.
# The calculation will start as soon as the requested resources are available.

# Open GAMESS output file:
if { $NPROCS > 1 } {
    if [ catch {open $env(PWD)/$JOB.stdout r} OUT] {

	set ERRMESS "Cannot open file $env(PWD)/$JOB.stdout Error reported:\n $OUT\n"
	$driver put -append yes $path $ERRMESS
	Rappture::result $driver
	exit 0

    } else {

	while { [gets $OUT line] >= 0 } { lappend lineList $line }
	close $OUT

    }

} else {
    set lineList [lrange [split $LOG \n] 0 end]

    set test [ catch {open $env(PWD)/$JOB.stdout w} OUT]
    foreach line $lineList {
	puts $OUT "$line"
    }
    close $OUT
}

regexp { NUMBER OF CARTESIAN GAUSSIAN BASIS FUNCTIONS .*?([0-9]+)} $lineList match NBS
regexp { TOTAL NUMBER OF ATOMS .*?([0-9]+)} $lineList match NA

#puts "My List: is [llength $lineList] records long.\n"
#after 10000;


#$driver put -append yes $path "\n\n\tNo. of atoms: $NA\n";
#Rappture::result $driver
#exit 0



#Read the atomic coordinates of the optimized structure:
if {$EXETYP == "Check"} {set RunTyp "Energy" }

switch $RunTyp {

    "Optimize" {
	for {set lineNum 0} {$lineNum <= [llength $lineList]} {incr lineNum} {
	    set line [lindex $lineList $lineNum]

	    # Geometry converged: read coordinates and Stop parsing $line
	    if {[regexp {EQUILIBRIUM GEOMETRY LOCATED} $line]} {
   while {![regexp {COORDINATES OF ALL ATOMS} [lindex $lineList $lineNum]]} {incr lineNum 1}
		incr lineNum 3
		break
	    }
	}

# Read the Optimum Geometry:
	for { set i 1} {$i <= $NA} {incr i} {
	    set line [lindex $lineList $lineNum]
	    
	    scan $line {%s %g %g %g %g}                     \
		MolStr($i.AtomName) MolStr($i.AtomicNumber) \
		MolStr($i.x) MolStr($i.y) MolStr($i.z)
	    incr lineNum

	}	
    }


    default {

	for {set lineNum 0} {$lineNum <= [llength $lineList]} {incr lineNum} {
	    set line [lindex $lineList $lineNum]

	    if {[regexp {COORDINATES \(BOHR\)} $line]} {
		incr lineNum 1
		break
	    }
	}


	for { set i 1} {$i <= $NA} {incr i} {
	    set line [lindex $lineList [expr {$lineNum + $i}]];

	    scan $line {%s %g %g %g %g}                     \
		MolStr($i.AtomName) MolStr($i.AtomicNumber) \
		MolStr($i.x) MolStr($i.y) MolStr($i.z)

# Transform A.U. (Bohrs)  to Angstroms:
	    set Bohr 0.529177249
	    set MolStr($i.x) [expr {$Bohr * $MolStr($i.x)}]
	    set MolStr($i.y) [expr {$Bohr * $MolStr($i.y)}]
	    set MolStr($i.z) [expr {$Bohr * $MolStr($i.z)}]

	}
    }
}



#BTA: write the Relevant data:
#1.- Geometry
set geom 0
for {set i 1} {$i <= $NA } {incr i} {
    if {[info exists MolStr($i.AtomName)]} {
        set id $MolStr($i.AtomName)
        if {![regexp {^[A-Za-z]+} $id symbol]} {
            set symbol ???
        }
        $driver put -append yes \
            output.structure(OptGeom).components.molecule.atom($i).symbol \
	    $symbol
        $driver put -append yes \
            output.structure(OptGeom).components.molecule.atom($i).xyz \
	    "$MolStr($i.x) $MolStr($i.y) $MolStr($i.z)"
        set geom 1
    }
}

if {$geom} {
    $driver put output.structure(OptGeom).about.description "Display of the molecular geometry"
    $driver put output.structure(OptGeom).about.label       "Molecular Geometry"
}

#2.- Log file
# $driver put -append yes output.log $lineList
set path "output.log.about.label"
$driver put -append yes $path "GAMESS logfile";
set path "output.log.about.description"
$driver put -append yes $path "This section contains the report of the calculation.\n"

set path "output.log"
foreach line $lineList {
    lappend TOMMP "$line\n"
    $driver put -append yes $path "$line\n"
}


#3.- Energy and Gradient

#4.- Mulliken analysis
#5.- Bond orders.


# 6.- Call wxmacmolplt:
# set status [catch {Rappture::exec wxmacmolplt $env(PWD)/$JOB.stdout   } result]
# set status [catch {Rappture::exec wxmacmolplt $env(PWD)/$JOB.stdout & } result]
# Rappture::exec wxmacmolplt $env(PWD)/$JOB.stdout
exec wxmacmolplt $env(PWD)/$JOB.stdout &

# clean up files...
#foreach name [glob -nocomplain $JOB.*] {
#    file delete -force $name
#}


# save the updated XML describing the run...
Rappture::result $driver

exit 0
